#!/usr/bin/env python
import xml.dom.minidom
from xml.dom.minidom import parse
from xml import xpath
from xml.dom.ext import PrettyPrint
import sys
import os.path
sys.path.append("../../../org.glite.deployment.config/src/scripts")
from gLiteInstallerLib import UltraPrettyPrint
from  gLiteInstallerLib import gLib


trans = {'file-transfer.DBPASSWORD':'transfer-agent.DBPassword',
         'file-transfer.DBUSER':'transfer-agent.DBUser',
         'file-transfer.DBHOST':'transfer-agent.DBHost',
         'file-transfer.DBSERVICENAME':'transfer-agent.DBService',
         'init.gid':'transfer-agent.group.gid',
         'init.uid':'transfer.agent.user.uid',
         'init.groupname':'transfer-agent.groupname',
         'init.username':'transfer-agent.username',
         'file-transfer.DBPORT':'transfer-agent.DBPort'}

def addInstance(dom, name, service, before=''):
    root = dom.documentElement
    nodes = xpath.Evaluate('config/instance[@name="%s"]' % name, dom)
    if len(nodes) > 0:
        print "Instance %s already exist" % name
    else:
            new = dom.createElement('instance')
            new.setAttributeNS('','name', name)
            new.setAttributeNS('','service', service)
            new.setAttributeNS('','description', 'Instance created by script')
            if before == '':
                root.appendChild(new)
            else:
                root.insertBefore(new, before)
    return new




if len(sys.argv) != 3:
	print "Usage:\nfta-config-migrator-1.3-1.4.py <config_file> <template>"
	sys.exit(1)

config = parse(sys.argv[1])
template = parse(sys.argv[2])

required_version = '2.1.0'
paramNode = xpath.Evaluate('/config/parameters', config)
if len(paramNode) > 1:
        # there can be only one parameters section
        print "problem ...."
        sys.exit(1)         

# Add DBBAckend
if len(xpath.Evaluate('//file-transfer.DBBackend', config)) > 0:
    print "file-transfer.DBBackend already exists"
else:
    new = config.createElement('file-transfer.DBBackend')
    new.setAttributeNS('','value', 'oracle')
    paramNode[0].appendChild(new) 

# Add fta.certificate.path
if len(xpath.Evaluate('//fta.certificate.path', config)) > 0:
    print "fta.certificate.path already exists"
else:
    new = config.createElement('fta.certificate.path')
    new.setAttributeNS('','value', '/etc/grid-security')
    paramNode[0].appendChild(new) 

# Add fta.certificate.path
if len(xpath.Evaluate('//transfer-agent.logdir', config)) > 0:
    print "fta.certificate.path already exists"
else:
    new = config.createElement('transfer-agent.logdir')
    new.setAttributeNS('','value', '${GLITE_LOCATION_LOG}')
    paramNode[0].appendChild(new) 


print "Importing instances"
instances = xpath.Evaluate('//instance[not (@service)]', config)
for instance in instances:
    nameNode = instance.getElementsByTagName('transfer-vo-agent.Name')
    if instance.getAttribute('service') == '':
        if nameNode != []:
            if instance.getElementsByTagName('transfer-vo-agent.DeploymentType')[0].getAttribute('value') == 'fts':
                serviceType = 'transfer-vo-agent-fts'
            else:
                serviceType = 'transfer-vo-agent-fps'
        else:
            nameNode = instance.getElementsByTagName('transfer-channel-agent.Name')
            serviceType = 'transfer-channel-agent-urlcopy'
        #print nameNode, serviceType

        name = nameNode[0].getAttribute('value')
        print name
        nameNode[0].parentNode.removeChild(nameNode[0])
        new = addInstance(config, name, serviceType, paramNode[0])
        new_params = instance.getElementsByTagName('parameters')[0].cloneNode('deep')
        new.appendChild(new_params)
        instance.parentNode.removeChild(instance)

# import service specific defaults
for param in xpath.Evaluate('//parameters[@service]', template):
    paramNode[0].parentNode.insertBefore(param.cloneNode('deep'), paramNode[0])
    
# rename parameters
for param in trans.keys():
    paramNodes = xpath.Evaluate('//%s' % param, config)
    for node in paramNodes:
        attrs = node.attributes
        print node.nodeName, attrs
        new = config.createElement(trans[param])
        for attribute in attrs.keys():
            new.setAttributeNS('',attribute, node.getAttribute(attribute))
        node.parentNode.insertBefore(new, paramNode[0].firstChild)
        node.parentNode.removeChild(node)
    # replace references
    for node in xpath.Evaluate('//*[contains(@value,"%s")]' % param, config):
        attr = node.getAttribute('value')
        attr = attr.replace(param, trans[param])
        node.setAttributeNS("", 'value', attr)
            
# Cleanup from not needed parameters
for param in ['transfer-agent.log.Filename','transfer-vo-agent.DeploymentType','file-transfer.VONAME',\
        'vo.name']:
    to_be_removed = xpath.Evaluate('//%s' % param, config)
    for rm in to_be_removed:
        rm.parentNode.removeChild(rm)
    # replace references
    for node in xpath.Evaluate('//*[contains(@value,"%s")]' % param, config):
        attr = node.getAttribute('value')
        attr = attr.replace("${%s}-" % param,"")
        attr = attr.replace("${%s}/" % param,"")
        attr = attr.replace("${%s}" % param,"")
        attr = attr.replace("%s" % param,"")
        node.setAttributeNS("", 'value', attr)    

c_file = os.path.basename(sys.argv[2])
if os.path.exists(c_file):
    os.rename(c_file, "%s.back" % c_file)

file = open(c_file, "w")
#PrettyPrint(root)
UltraPrettyPrint(config, stream=file)
file.close()
